"""Tests for `snapchat_dl` package."""
import os
import shutil
import unittest

from requests.exceptions import HTTPError

from snapchat_dl.downloader import download_url


class Test_downloader(unittest.TestCase):
    """Tests for `snapchat_dl.downloader.download_url` package."""

    def setUp(self):
        """Set up test fixtures."""
        self.test_url = "http://commondatastorage.googleapis.com/gtv-videos-bucket/sample/ForBiggerJoyrides.mp4"
        self.test_url404 = "https://google.com/error.html"
        self.username = "invalidusername"

    @classmethod
    def tearDownClass(cls):
        """Tear down test fixtures."""
        for file in ["test_dl_23.mp4", "test_dl_23.html"]:
            if os.path.isfile(file):
                os.remove(file)

    def test_download_url(self):
        """Test snapchat_dl download_url."""
        with open("test_dl_23.mp4", "a") as f:
            f.close()
        download_url(self.test_url, "test_dl_23", sleep_interval=0)

    def test_download_url_raise(self):
        """Test snapchat_dl download_url with invalid url."""
        with self.assertRaises(HTTPError):
            download_url(self.test_url404, "test_dl_23", sleep_interval=0)



"""Utility functions for snapchat_dl."""
import json
import os
import re
from argparse import Namespace
from datetime import datetime

from loguru import logger


class NoStoriesAvailable(Exception):
    """Exception when there are no stories."""

    pass


def strf_time(timestamp, format_str):
    """Format unix timestamp to custom format.

    Args:
        timestamp (int): unix timestamp
        format_str (str): valid python date time format

    Returns:
        str: timestamp formatted to custom format.
    """
    return datetime.utcfromtimestamp(timestamp).strftime(format_str)


def valid_username(username):
    """Validate Username.

    Args:
        username (str): Snapchat Username

    Returns:
        bool: True if username is valid.
    """
    match = re.match(r"(?P<username>^[\-\w\.\_]{3,15}$)", username)
    if match is None:
        return False

    return match and match.groupdict()["username"] == username


def search_usernames(string: str) -> list:
    """Return list of usernames found in a string.

    Args:
        string (str): string to search for usernames

    Returns:
        list: usernames found in string
    """
    return list(
        sorted(
            set(
                [
                    username
                    for username in re.findall(
                        r"https?://(?:story|www).snapchat.com/(?:[suad]+/|@)([\-\w\.\_]{3,15})",
                        string,
                    )
                    if valid_username(username)
                ]
            )
        )
    )


def use_batch_file(args: Namespace) -> list:
    """Return list of usernames from file args.batch_file.

    Args:
        args (Namespace): argparse Namespace

    Raises:
        os.error: raises if batch_file not found

    Returns:
        list: usernames read from batch_file
    """
    usernames = list()
    if args.batch_file is not None:
        if os.path.isfile(args.batch_file) is False:
            raise Exception(
                logger.error("Invalid Batch File at {}".format(args.batch_file))
            )

        with open(args.batch_file, "r") as f:
            for u in f.read().split("\n"):
                username = u.strip()
                if valid_username(username) and username not in usernames:
                    usernames.append(username)

    return usernames


def use_prefix_dir(args: Namespace):
    """Return dirnames as username from file args.scan_prefix.

    Args:
        args (Namespace): argparse Namespace

    Returns:
        list: usernames read from scan_prefix
    """
    usernames = list()
    if args.scan_prefix:
        for username in [
            o
            for o in os.listdir(args.save_prefix)
            if os.path.isdir(os.path.join(args.save_prefix, o))
        ]:
            if username not in usernames and valid_username(username):
                usernames.append(username)

        if args.quiet is False:
            logger.info(
                "Added {} usernames from {}".format(len(usernames), args.save_prefix)
            )

    return list(sorted(set(usernames)))


def dump_text_file(content: str, filepath: str):
    """Write content to filepath using `tx` mode.

    Args:
        content (str): File content to write.
        filepath (str): Filepath.

    This will overwrite the file.
    """
    dirpath = os.path.dirname(filepath)

    os.makedirs(dirpath, exist_ok=True)

    if not os.path.isfile(filepath):
        with open(filepath, "w+") as f:
            f.write(content)


def dump_response(content: dict, path: str):
    """Save JSON file

    Args:
        content: JSON data
        path: Path to save json

    Returns:
        None
    """
    dump_text_file(json.dumps(content), path)



#!/usr/bin/env python
"""Tests for `snapchat_dl` package."""
import unittest
from argparse import Namespace

from snapchat_dl.utils import search_usernames
from snapchat_dl.utils import strf_time
from snapchat_dl.utils import use_batch_file
from snapchat_dl.utils import use_prefix_dir
from snapchat_dl.utils import valid_username


class Test_utils(unittest.TestCase):
    """Tests for `snapchat_dl` package."""

    def test_valid_username(self):
        """Test for invalid username."""
        self.assertFalse(valid_username("2323 2323"))

    def test_strf_time(self):
        """Test strf_time."""
        self.assertEqual(
            strf_time(978307200, "%Y-%m-%dT%H-%M-%S"), "2001-01-01T00-00-00",
        )

    def test_search_usernames(self):
        """Test usernames search in string."""
        string = """
            https://story.snapchat.com/s/in#invalidusername
            https://story.snapchat.com/s/username1
            https://story.snapchat.com/s/user.name2
            https://story.snapchat.com/s/user_name
            https://story.snapchat.com/@user_name
        """
        usernames = ["user.name2", "user_name", "username1"]
        self.assertListEqual(search_usernames(string), usernames)

    def test_use_batch_file(self):
        args = Namespace(batch_file="tests/mock_data/batch_file.txt")
        usernames = ["username1", "user.name2", "user_name"]
        self.assertListEqual(use_batch_file(args), usernames)

    def test_use_batch_file_err(self):
        args = Namespace(batch_file="tests/mock_data/batch_file_err.txt")
        with self.assertRaises(Exception):
            use_batch_file(args)

    def test_use_prefix_dir(self):
        args = Namespace(scan_prefix=True, save_prefix="tests/mock_data", quiet=False)
        usernames = ["user.1name", "user1"]
        self.assertListEqual(use_prefix_dir(args), usernames)



#!/usr/bin/env python
"""Tests for `snapchat_dl` package."""
import json
import os
import shutil
import unittest
from unittest import mock

from snapchat_dl.snapchat_dl import SnapchatDL
from snapchat_dl.utils import NoStoriesAvailable


def teardown_module(module):
    os.system("rm -rf .test-data")


class TestSnapchat_dl(unittest.TestCase):
    """Tests for `snapchat_dl` package."""

    def setUp(self):
        """Set up test fixtures."""
        self.snapchat_dl = SnapchatDL(
            limit_story=10, quiet=True, directory_prefix=".test-data", dump_json=True,
        )
        self.test_url = "http://commondatastorage.googleapis.com/gtv-videos-bucket/sample/ForBiggerJoyrides.mp4"
        self.test_url404 = "https://google.com/error.html"
        self.username = "invalidusername"

    def test_class_init(self):
        """Test snapchat_dl init."""
        self.assertTrue(self.snapchat_dl)

    def test_get_stories_no_stories(self):
        """Test snapchat_dl Stories are not available."""
        with self.assertRaises(NoStoriesAvailable):
            self.snapchat_dl.download("username")

    @mock.patch("snapchat_dl.snapchat_dl.SnapchatDL._api_fetch_story")
    def test_get_stories_ok(self, fake_get):
        """Test snapchat_dl Download."""

        class Mock_Response:
            status_code = 200

            def json(self) -> dict:
                with open(
                    "tests/mock_data/invalidusername.json", "r", encoding="utf8"
                ) as f:
                    return json.load(f)

        fake_get.return_value = Mock_Response()
        self.snapchat_dl.download(self.username)



"""The Main Snapchat Downloader Class."""
import concurrent.futures
import os
import re

import requests
from loguru import logger

from snapchat_dl.downloader import download_url
from snapchat_dl.utils import dump_response
from snapchat_dl.utils import NoStoriesAvailable
from snapchat_dl.utils import strf_time


class SnapchatDL:
    """Interact with Snapchat API to download story."""

    def __init__(
        self,
        directory_prefix=".",
        max_workers=2,
        limit_story=-1,
        sleep_interval=1,
        quiet=False,
        dump_json=False,
    ):
        self.directory_prefix = os.path.abspath(os.path.normpath(directory_prefix))
        self.max_workers = max_workers
        self.limit_story = limit_story
        self.sleep_interval = sleep_interval
        self.quiet = quiet
        self.dump_json = dump_json
        self.endpoint = "https://storysharing.snapchat.com/v1/fetch/{}?request_origin=ORIGIN_WEB_PLAYER"
        self.reaponse_ok = requests.codes.get("ok")

    def _api_fetch_story(self, username):
        """Download user stories.

        Args:
            username (str): Snapchat `username`

        Returns: (requests.Response): response
        """
        api_url = self.endpoint.format(username)
        response = requests.get(api_url)

        return response

    def parse_snap_user(self, response):
        """Generate userInfo json object from story response.

        Args:
            response (dict): Story response

        Returns:
            dict: userInfo object.
        """
        userInfo = {"id": response["id"]}
        for key in list(["emoji", "title"]):
            userInfo[key] = response["metadata"][key]

        return userInfo

    def download(self, username):
        """Download Snapchat Story for `username`.

        Args:
            username (str): Snapchat `username`

        Returns:
            [bool]: story downloader
        """
        response = self._api_fetch_story(username)
        if response.status_code != 200:
            if self.quiet is False:
                logger.info("\033[91m{}\033[0m has no stories".format(username))
            raise NoStoriesAvailable

        resp_json = response.json()
        snap_user = self.parse_snap_user(resp_json.get("story"))
        stories = resp_json.get("story").get("snaps")
        if self.limit_story > -1:
            stories = stories[0 : self.limit_story]

        logger.info("[+] {} has {} stories".format(username, len(stories)))

        executor = concurrent.futures.ThreadPoolExecutor(max_workers=self.max_workers)
        try:
            for media in stories:
                snap_id = media["id"]
                media_url = media["media"]["mediaUrl"]
                overlay_url = (
                    media["overlayImage"]["mediaUrl"]
                    if media["media"]["type"] is "VIDEO"
                    else ""
                )
                timestamp = int(media["captureTimeSecs"])
                date_str = strf_time(timestamp, "%Y-%m-%d")

                dir_name = os.path.join(self.directory_prefix, username, date_str)

                filename = strf_time(timestamp, "%Y-%m-%d_%H-%M-%S {} {}").format(
                    snap_id, username
                )

                if self.dump_json:
                    filename_json = os.path.join(dir_name, filename + ".json")
                    media_json = dict(media)
                    media_json["snapUser"] = snap_user
                    dump_response(media_json, filename_json)

                media_output = os.path.join(dir_name, filename)
                executor.submit(
                    download_url, media_url, media_output, self.sleep_interval
                )

                if len(overlay_url) > 0:
                    overlay_output = os.path.join(dir_name, filename + "_ol")
                    executor.submit(
                        download_url, overlay_url, overlay_output, self.sleep_interval
                    )

        except KeyboardInterrupt:
            executor.shutdown(wait=False)

        logger.info("[✔] {} stories downloaded".format(username, len(stories)))



#!/usr/bin/env python
"""The setup script."""
from setuptools import find_packages
from setuptools import setup


version = {}
with open("snapchat_dl/version.py") as fp:
    exec(fp.read(), version)

with open("README.md") as readme_file:
    readme = readme_file.read()

requirements = list()

with open("requirements.txt", "r") as file:
    requirements = [r for r in file.readlines() if len(r) > 0]

test_requirements = ["pytest"].extend(requirements)

setup(
    name="snapchat-dl",
    version=version["__version__"],
    description="Snapchat Public Stories Downloader.",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/skyme5/snapchat-dl",
    author="Aakash Gajjar",
    author_email="skyqutip@gmail.com",
    entry_points={"console_scripts": ["snapchat-dl=snapchat_dl.app:main",],},
    include_package_data=True,
    install_requires=requirements,
    test_suite="tests",
    tests_require=test_requirements,
    python_requires=">=3.5",
    keywords="snapchat-dl",
    license="MIT license",
    packages=find_packages(include=["snapchat_dl", "snapchat_dl.*"]),
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)
